###############################################################################
# Perform analysis via non-linear modelling to determine which
#  buffers differ from each other in rate for which responses, 
#  temps, etc.
#
# Jul 7, 2010
#
# Author: McVey
###############################################################################

##############################################
## Setup

require(ggplot2)
require(nlme)
require(doBy)
require(RUnit)
require(gdata)

source('Tools/newPlotFunctions.R')

##############################################
## Get data

attach('rdata/reactionData.RData')

dim(longDf)
# [1] 783   6

head(longDf)
#   Replicate Buffer Time Temperature Response Concentration
# 1         1      A    0           4   Ianite          3.23
# 2         1      A    0           4  Zerbene          3.87
# 3         1      A    0           4       LL          3.37
# 4         1      A    1           4   Ianite          3.15
# 5         1      A    1           4  Zerbene          3.86
# 6         1      A    1           4       LL          3.40

if(nrow(longDf)!=783) stop('Data dimensions have changed.  Recommend running code step-by-step.')

#############################################################################
## Make sure data is ready to fit

table(is.na(longDf$Concentration), longDf$Buffer)
#        
#           A   B   C
#   FALSE 259 261 260
#   TRUE    2   0   1

##remove NA rows
longDf <- subset(longDf, !is.na(longDf$Concentration))

dim(longDf)
# [1] 780   6


#############################################################################
## Exponential fits to individual curves

########
##Fit the exponential model to each curve and return a variable with the fitted values

fitExp <- function(df) {
	
	startInit <- 3.2
	expFit <- try(nls(Concentration ~ init*exp(-rate*Time), 
					data=df, start=list(init=startInit, rate=.01)))
	
	df$expFitted <- fitted(expFit)
	
	return(df)
	
}

#TODO:

expFitDf <- ddply(longDf, .(Response, Buffer, Temperature), fitExp)

dim(expFitDf)
# [1] 780   7

#########
##Plot the observed data vs the fitted curves to assess model fit
checkFitPlot <- ggplot(data=expFitDf,
				aes(x=Time, y=Concentration)) +
		facet_grid(Temperature ~ Buffer) +
		geom_point() + geom_smooth(color='purple', se=FALSE) +
		geom_point(aes(y=expFitted), color='red') + geom_line(aes(y=expFitted), color='red')

cFp1 <- checkFitPlot %+% subset(expFitDf, Response=='Ianite') + ylab('Ianite')
cFp2 <- checkFitPlot %+% subset(expFitDf, Response=='Zerbene') + ylab('Zerbene')
cFp3 <- checkFitPlot %+% subset(expFitDf, Response=='LL') + ylab('LL')



########
##Fit the exponential model (same as above in fitExp), but return only the rate and init (i.e. summarized df)
getExpRate <- function(df) {
	
	## We need a different starting value for the voltage models to avoid crashing
	startInit <- 3.2
	
	expFit <- try(nls(Concentration ~ init*exp(-rate*Time), 
					data=df, 
					start=list(init=startInit, rate=.01)))
	
	return(c(coef(expFit)[2], coef(expFit)[1]))
	
}

expRateDf <- ddply(longDf, .(Response, Buffer, Temperature), getExpRate)

dim(expRateDf)
# [1] 36  5

if(nrow(expRateDf)!=36) stop('Number of conditions has changed.  Recommend running code step-by-step.')

head(expRateDf)
#   Response Buffer Temperature        rate     init
# 1   Ianite      A           4 0.003475422 3.166677
# 2   Ianite      A          25 0.008250778 3.105775
# 3   Ianite      A          37 0.023556515 3.113208
# 4   Ianite      A          45 0.032476902 3.089172
# 5   Ianite      B           4 0.004945252 3.151798
# 6   Ianite      B          25 0.010612746 3.074853


#########
##Plot rates across temps for each Buffer and Response
rateTemp <- ggplot(data=expRateDf,
				aes(x=Temperature, y=rate, color=Buffer)) +
		facet_wrap( ~ Response, scales='free_y') + geom_point() + geom_line()



#######################################################################
## Fitting an overall model to allow for stat comparison of rates by Buffer
## To do this with known tools, need to linearize the model

##This will be useful in several places . . . 
numLevels <- length(levels(longDf$Buffer))

########
##Make contrasts matrix

##Determine number of pairwise comparisons to be made
numComps <- factorial(numLevels)/((factorial(numLevels-2)*2))

##Make a matrix with 4 choose 2 rows and 4+1 columns
##For all pairwise comparisons of rate by Buffer
allContrasts <- matrix(0, nrow=numComps, ncol=numLevels+1)
dimnames(allContrasts)[[1]] <- rep(NA, times=numComps)

doneRows=0
col=2
for(j in (numLevels-1):1) {
	for(i in 1:j) {
		
		allContrasts[i+doneRows,col] <- -1 
		
		allContrasts[i+doneRows,col+i] <- 1
		
		dimnames(allContrasts)[[1]][i+doneRows] <- paste(as.character(levels(longDf$Buffer)[col-1+i]),'-',as.character(levels(longDf$Buffer)[col-1]))
	}
	doneRows <- sum(doneRows, j)
	col <- col + 1
}

#########
##Fit the linearized model with a separate slope estimate for each level of Buffer
##Return the fitted values
predictRateModel <- function(df) {
	
	rateFit <- lm(log(Concentration) ~ Time:Buffer, data=df)
	
	esticon(rateFit, allContrasts)
	
	df <- subset(df, !is.na(Concentration))
	
	df$FitConc <- exp(rateFit$fitted)
	
	return(df)	
	
}

fitPredictDf <- ddply(longDf, .(Response, Temperature), predictRateModel)

dim(fitPredictDf)
# [1] 780   7

head(fitPredictDf)
#   Replicate Buffer Time Temperature Response Concentration  FitConc
# 1         1      A    0           4   Ianite          3.23 3.171951
# 2         1      A    1           4   Ianite          3.15 3.160474
# 3         1      A    2           4   Ianite          3.13 3.149038
# 4         1      A    3           4   Ianite          3.10 3.137643
# 5         1      A    5           4   Ianite          3.11 3.114977
# 6         1      A    8           4   Ianite          3.04 3.081285


#########
##Plot observed data against fitted values for assessment of fit on log scale
##Note: on this scale, we expect both to be linear
checkLinFit <- ggplot(data=subset(fitPredictDf, Response=='Ianite'),
				aes(x=Time, y=log(Concentration))) + facet_grid(Temperature ~ Buffer) + errBars(summary='meanSE') +
		geom_line(aes(y=log(FitConc)), color='red') + geom_point(aes(y=log(FitConc)), color='red')

cF1p1 <- checkLinFit + ylab('Ianite')
cF1p2 <- checkLinFit %+% subset(fitPredictDf, Response=='Zerbene') + ylab('Zerbene')
cF1p3 <- checkLinFit %+% subset(fitPredictDf, Response=='LL') + ylab('LL')

#########
##Plot observed data against fitted values for assessment of fit on original scale
checkLinFit2 <- ggplot(data=subset(fitPredictDf, Response=='Ianite'),
				aes(x=Time, y=Concentration)) + facet_grid(Temperature ~ Buffer) + errBars(summary='meanSE') +
		geom_line(aes(y=FitConc), color='red') + geom_point(aes(y=FitConc), color='red')

cF2p1 <- checkLinFit2 + ylab('Ianite')
cF2p2 <- checkLinFit2 %+% subset(fitPredictDf, Response=='Zerbene') + ylab('Zerbene')
cF2p3 <- checkLinFit2 %+% subset(fitPredictDf, Response=='LL') + ylab('LL')

#########
##Overlay fits from each buffer for a given response, facet on temp
overlayFits <- ggplot(data=subset(fitPredictDf, Response=='Ianite'),
		aes(x=Time, y=FitConc, color=Buffer)) + facet_wrap( ~ Temperature) + geom_point() + geom_line()

overlayp1 <- overlayFits + ylab('Ianite')
overlayp2 <- overlayFits %+% subset(fitPredictDf, Response=='Zerbene') + ylab('Zerbene') 
overlayp3 <- overlayFits %+% subset(fitPredictDf, Response=='LL') + ylab('LL')


#########
##Fit linearized model (same as above) and return just rate (i.e. summarized df)
fitRateModel <- function(df) {
	
	rateFit <- lm(log(Concentration) ~ Time:Buffer, data=df)
	
	rates <- -signif(coef(rateFit)[2:length(coef(rateFit))],2)
	
	names(rates) <- gsub('Time:Buffer','', names(rates))
	
	return(rates)
}

fitRatesDf <- ddply(longDf, .(Response, Temperature), fitRateModel)

meltDf <- melt(fitRatesDf, id=c('Response','Temperature'), measured=c('Vial','Accuchek','Animas','BD','Minimed','Quick-set','Rapid D'))

names(meltDf) <- c('Response','Temperature','Buffer','rate')

modelRatesDf <- meltDf

dim(modelRatesDf)
# [1] 36  4

if(nrow(modelRatesDf)!=36) stop('Melted data dimensions have changed.  Recommend running code step-by-step.')

head(modelRatesDf)
#   Response Temperature Buffer     rate
# 1   Ianite           4      A  3.6e-03
# 2   Ianite          25      A  8.3e-03
# 3   Ianite          37      A  2.3e-02
# 4   Ianite          45      A  3.3e-02
# 5  Zerbene           4      A  1.8e-04
# 6  Zerbene          25      A -4.3e-06


########
##Display rate results
rateCompPlot <- ggplot(data=subset(modelRatesDf, Response=='Ianite'),
		aes(x=Buffer, y=rate)) + geom_point() + facet_wrap( ~ Temperature, scales='free')

rateCompp1 <- rateCompPlot + ylab('Ianite')
rateCompp2 <- rateCompPlot %+% subset(modelRatesDf, Response=='Zerbene') + ylab('Zerbene')
rateCompp3 <- rateCompPlot %+% subset(modelRatesDf, Response=='LL') + ylab('LL')

compareRates <- function(df) {
	
	rateFit <- lm(log(Concentration) ~ Time:Buffer, data=df)
	
	omnibusP <- summary.aov(rateFit)[[1]][[5]][1]
	
	contrasts <- esticon(rateFit, allContrasts)
	
	contrasts$Comparison <- row.names(contrasts)
	contrasts$NS <- contrasts$Pr>.05
	
	##If omnibus is non-sig, set all contrasts non-sig
	if(omnibusP>.05) contrasts$NS <- TRUE
	
	return(contrasts)	
	
}

contrastsDf <- ddply(longDf, .(Response, Temperature), compareRates)

dim(contrastsDf)
# [1] 36 12

head(contrastsDf)
#   Response Temperature beta0     Estimate    Std.Error   t.value DF Pr(>|t|)
# 1   Ianite           4     0 -0.001941153 0.0004665339 -4.160798 68 9.11e-05
# 2   Ianite           4     0  0.003360765 0.0004665339  7.203689 68 0.00e+00
# 3   Ianite           4     0  0.005301919 0.0004665339 11.364487 68 0.00e+00
# 4   Ianite          25     0 -0.003418250 0.0004604285 -7.424062 59 0.00e+00
# 5   Ianite          25     0  0.006322159 0.0004604285 13.731034 59 0.00e+00
# 6   Ianite          25     0  0.009740409 0.0004604285 21.155096 59 0.00e+00
#       Lower.CI     Upper.CI Comparison    NS
# 1 -0.002872107 -0.001010200      B - A FALSE
# 2  0.002429811  0.004291719      C - A FALSE
# 3  0.004370965  0.006232872      C - B FALSE
# 4 -0.004339565 -0.002496934      B - A FALSE
# 5  0.005400844  0.007243475      C - A FALSE
# 6  0.008819094  0.010661724      C - B FALSE



contrastsDf$Large.01 <- NA
contrastsDf$Large.02 <- NA

for(i in 1:nrow(contrastsDf)) {
	contrastsDf$Large.01[i] <- min(abs(contrastsDf$Lower.CI[i]), abs(contrastsDf$Upper.CI[i]))>.01
	contrastsDf$Large.02[i] <- min(abs(contrastsDf$Lower.CI[i]), abs(contrastsDf$Upper.CI[i]))>.02
}

##Assign level of significance (Note: this must be done in order)
contrastsDf$Level <- NA
contrastsDf$Level[contrastsDf$Large.01] <- '>0.01'
contrastsDf$Level[contrastsDf$Large.02] <- '>0.02'
contrastsDf$Level[contrastsDf$NS] <- 'NS'
contrastsDf$Level[!contrastsDf$NS&!contrastsDf$Large.01] <- '>0.0'

reportDf <- contrastsDf[,c('Response','Temperature','Comparison','Estimate','Lower.CI','Upper.CI','Level')]

dim(reportDf)
# [1] 36  7

head(reportDf)
#   Response Temperature Comparison     Estimate     Lower.CI     Upper.CI Level
# 1   Ianite           4      B - A -0.001941153 -0.002872107 -0.001010200  >0.0
# 2   Ianite           4      C - A  0.003360765  0.002429811  0.004291719  >0.0
# 3   Ianite           4      C - B  0.005301919  0.004370965  0.006232872  >0.0
# 4   Ianite          25      B - A -0.003418250 -0.004339565 -0.002496934  >0.0
# 5   Ianite          25      C - A  0.006322159  0.005400844  0.007243475  >0.0
# 6   Ianite          25      C - B  0.009740409  0.008819094  0.010661724  >0.0


table(reportDf$Level)
# 
#  >0.0 >0.01 >0.02    NS 
#    21     4     2     9 


##This makes a nice table for pairwise comparison results 
makeCompTable <- function(df) {
	
	##Setup a dataframe with a pairwise table structure
	tableDf <- as.data.frame(matrix(NA, ncol=numLevels, nrow=numLevels))
	row.names(tableDf) <- levels(longDf$Buffer)
	names(tableDf) <- levels(longDf$Buffer)
	
	##Fill the table with the right results
	for(i in 1:numLevels) {
		for(j in 1:numLevels) {
			index <- intersect(grep(names(tableDf)[i], df$Comparison), grep(row.names(tableDf)[j], df$Comparison))
			if(length(index)==1) tableDf[i,j] <- df$Level[index] else tableDf[i,j] <- '-'
		}
	}
	
	return(tableDf)
}

makeCompTable(df=subset(reportDf, Temperature==45 & Response=='Ianite'))
#       A     B     C
# A     -  >0.0 >0.01
# B  >0.0     - >0.02
# C >0.01 >0.02     -



tab4I <- makeCompTable(df=subset(reportDf, Temperature==4 & Response=='Ianite'))
tab25I <- makeCompTable(df=subset(reportDf, Temperature==25 & Response=='Ianite'))
tab37I <- makeCompTable(df=subset(reportDf, Temperature==37 & Response=='Ianite'))
tab45I <- makeCompTable(df=subset(reportDf, Temperature==45 & Response=='Ianite'))
tab4Z <- makeCompTable(df=subset(reportDf, Temperature==4 & Response=='Zerbene'))
tab25Z <- makeCompTable(df=subset(reportDf, Temperature==25 & Response=='Zerbene'))
tab37Z <- makeCompTable(df=subset(reportDf, Temperature==37 & Response=='Zerbene'))
tab45Z <- makeCompTable(df=subset(reportDf, Temperature==45 & Response=='Zerbene'))
tab4L <- makeCompTable(df=subset(reportDf, Temperature==4 & Response=='LL'))
tab25L <- makeCompTable(df=subset(reportDf, Temperature==25 & Response=='LL'))
tab37L <- makeCompTable(df=subset(reportDf, Temperature==37 & Response=='LL'))
tab45L <- makeCompTable(df=subset(reportDf, Temperature==45 & Response=='LL'))

write.csv(reportDf, 'output/Statistical Analysis Contrast Results.csv', row.names=FALSE)



###################################################################################
## Check results

##Compare rates from individual exponential fits to rates from linearized model
##The individual exp fits should be the "gold standard" since we can see that they fit well
##Close agreement between the two sets of rates indicates that the linearization is working well

compareExptoLinRatePlot <- ggplot(data=expRateDf,
				aes(x=Temperature, y=rate)) +
		facet_wrap( ~ Buffer + Response, scales='free') + geom_point() + geom_line() +
		geom_point(data=modelRatesDf, color='red') + geom_line(data=modelRatesDf, color='red')



